/* *********************************************************************** */ 
/*                          Smus2sco v2.0                                  */
/*                       1990 Jean-Francois Giorgi                         */
/* *********************************************************************** */ 

#include "smus2sco.h"

/* ********************************* */
/* analsmus.c : analize a SMUS score */
/* ********************************* */

BOOL WarnINDef;       /* true if it exists notes without instruments */
UWORD shnote;         /* shortest note (its index in Durations[])    */
UWORD shtrinote;      /* shortest triolet note (idem)                */
UWORD shdotnote;      /* shortest dotted note  (idem)                */
BOOL ftri;            /* TRUE if there are triplet notes             */
BOOL fdot;            /* TRUE if there are dotted note(s)            */
ULONG NUsages[64];    /* for each note.duration ,its #               */
WORD IRegs[MAX_INST]; /* instrument mapping                          */

char NoteDurMess[8][40] = {
  "normal notes     ",
  "dotted notes     ",
  "triplet notes    ",
  "dotted triplet   ",
  "quintuplet       ",
  "dotted quintuplet",
  "septuplet        ",
  "dotted septuplet "
};
/* *********************************************************************** */       
/* set the current instrument */
VOID Maj_Instr(in)
UBYTE in;
{
  WORD ic;

  ic = IRegs[in];
  if (ic == NO_INSTR) 
    printf("WARNING : Maj_Instr: unmapped instrument register %d\n",(WORD)in);
  else 
    InstrList->its_cour = ic;
}
/* *********************************************************************** */       
/* treatement of a SEvent */
WORD Anal_SEvent(se)
SEvent se;
{
  register UBYTE sid,tone;
  SNote *sn;
  SMNote *sm;
  WORD ic = InstrList->its_cour;


  sn = (SNote *) (&se);
  sm = (SMNote *) (&se);
  sid = se.sID;


  if (IsNote(sid)) {
    tone = sn->tone;
    if (ic == NO_INSTR) {
      WarnINDef = TRUE; /* there is a note without any instrument */
    }
    else { /* set the usage of the current instrument */
      InstrList->its_Used[ic]    = TRUE;
      InstrList->its_MinTone[ic] = MIN(InstrList->its_MinTone[ic],tone);
      InstrList->its_MaxTone[ic] = MAX(InstrList->its_MaxTone[ic],tone);
    }
  }  
  else switch (sid) {
    case SID_Rest        : break;
    case SID_Instrument  : Maj_Instr(se.data); break;
    case SID_TimeSig     : break;
    case SID_KeySig      : break;
    case SID_Dynamic     : break;
    case SID_Clef        : break;
    case SID_Tempo       : break;
    case SID_MIDI_Chnl   : printf("MIDI channel %d\n",(int)se.data); break;
    case SID_MIDI_Preset : printf("MIDI preset %d\n",(int)se.data); break;
    default              : 
      if (sid>=144 && sid<=159)
        printf("Instant Music SEvents : %d\n",(int)sid); 
      else
        printf("unknow event : %d\n",(int)sid);
      break;
  }

  if (AsDuration(sid) && (!sm->chord)) {
    shnote   = (UWORD) (MINDUR(shnote,sm->duration));
    if (sn->nTuplet == 1) {
      shtrinote   = (UWORD) (MINDUR(shtrinote,sm->duration));
      ftri = TRUE;
    }
    if (sn->dot == 1) {
      shdotnote  = (UWORD) (MINDUR(shdotnote,sm->duration));
      fdot = TRUE;
    }
    NUsages[sm->duration] +=1;
    return(sm->duration);
  }
  else return(-1);
}
/* *********************************************************************** */       
/*
   play a track to : 
     - calculate total duration.
     - calculate instruments usage.
*/
VOID Anal_Track(tr,numt)
struct Track *tr;
UBYTE numt;
{
  register LONG pos;
  WORD duration;
  LONG tdur;

  tdur = 0L;
  duration = 0;

  InstrList->its_cour = (DefTI==TRUE) ? IRegs[numt] : NO_INSTR;
  /* starting instrument of this track (instrument numt is -d option) */

  for (pos=0;pos<(tr->tr_len);pos++) {
    duration = Anal_SEvent(tr->tr_fse[pos]);
    if (duration != -1) {
      tdur    += (LONG) (Durations[duration]);
    }
  }

  printf("Track %d as a duration of %ld 128th divisions\n",numt,tdur);
  tr->tr_duration = tdur;
}

/* *********************************************************************** */       
/* play the SMUS score to :
    - calculate total duration 
    - calculate instruments usage
*/
VOID Anal_Score()
{
  UBYTE track_cour;
  WORD ic,nc;
  UBYTE it,at;
  ULONG duration;

  WarnINDef = FALSE;
  duration  = 0L;
  shnote    = LONG_NOTE; /* our default base note          */
  shtrinote = 0;         /* 0 is index of whole note (1/1) */ 
  shdotnote = 0;         /* 0 is index of whole note (1/1) */ 
  ftri      = FALSE;
  fdot      = FALSE;
  for(nc=0;nc<64;nc++) NUsages[nc] = 0;

  for(track_cour=0;track_cour<The_Score->sc_ntc;track_cour++) {
    Anal_Track(The_Score->sc_tracks[track_cour],track_cour);
    duration = (ULONG) MAX(duration,The_Score->sc_tracks[track_cour]->tr_duration);
  }

  shnote    = shnote & noteDMask;    /* suppress NTuplet & dot bits */

  if (AnalOnly == TRUE) printf("Shortest note =  %d\n",shnote);
  if (ftri) {
    if (AnalOnly == TRUE) printf("Shortest triplet note = %d\n",shtrinote);
    if (AnalOnly == TRUE) printf("BaseRange = 3\n");
    BaseRange = 3;
  }
  else BaseRange = 1;


  if (fdot) {
    shdotnote = shdotnote & noteDMask; /* suppress NTuplet & dot bits */
    if (AnalOnly == TRUE) printf("Shortest dotted note  =  %d\n",shdotnote);
    if (shdotnote >= shnote) {
      if (AnalOnly == TRUE) printf("HALF TIME\n");
      shnote = MIN(shdotnote+1,7);
    }
  }

  shnote = MAX(shnote,2); /* 2 is quarter note */

  BaseDuration = Durations[shnote];

  if (AnalOnly == TRUE) printf("choosen BaseDuration = %d (%d)\n",BaseDuration,shnote);

  duration  = duration * BaseRange; /* should check overflow */
  duration  = duration / BaseDuration;  
  
  if (WarnINDef) {
    printf("\nWARNING THERE ARE NOTES WITHOUT ANY INSTRUMENT\n");
    printf("THEY WILL BE PLAYED WITH 'Default Instrument'\n");
  }

  The_Score->sc_duration = duration;
  printf("\nScore duration is %d beats \n",The_Score->sc_duration);

  printf("\nThere is %d Instruments\n",InstrList->its_num);
  printf("\nInstruments ranges :\n");
  for (ic=0;ic<InstrList->its_num;ic++)
    if (InstrList->its_Used[ic])
      printf("%s : %d to %d\n",InstrList->its_instrs[ic]->name,
                               InstrList->its_MinTone[ic],InstrList->its_MaxTone[ic]);
    else {
      printf("%s : is never used so i remove it\n",InstrList->its_instrs[ic]->name);
      FillZero(InstrList->its_instrs[ic]->name,strlen(InstrList->its_instrs[ic]->name));
    }

  for(nc=0;nc<64;nc++) {
    duration = ((ULONG)Durations[nc]) * ((ULONG)BaseRange);
    CDurations[nc] = (UWORD)(duration/((ULONG)BaseDuration));
  }

  if (AnalOnly==TRUE) {
    printf("\n------------------------------- notes usage table -----------------------------------\n");
    printf("|   1   |  1/2  |  1/4  |  1/8  | 1/16  | 1/32  | 1/64  | 1/128 | divisions         |\n");
    printf("|---------------------------------------------------------------|-------------------|\n");
    for(nc=0;nc<64;nc++) {
      printf("|%7d",NUsages[nc]);
      if ((nc%8) == 7) printf("| %s |\n",NoteDurMess[nc/8]);
    }
    printf("-------------------------------------------------------------------------------------\n");
  }

}
