/* *********************************************************************** */ 
/*                           Smus2Sco v2.0                                 */
/*                       1990 Jean-Francois Giorgi                         */
/* *********************************************************************** */ 

#include "smus2sco.h"

/* ******************************* */
/* convert.c : conversion routines */
/* ******************************* */

/* *********************************************************************** */

UWORD Durations[64] = {
/*   1     1/2    1/4   1/8   1/16  1/32  1/64  1/128  <- division          */
  0x6900,0x3480,0x1a40,0x0d20,0x690,0x348,0x1a4,0x0d2, /* normal notes      */
  0x9d80,0x4ec0,0x2760,0x13b0,0x9d8,0x4ec,0x276,0x13b, /* dotted notes      */
  0x4600,0x2300,0x1180,0x08c0,0x460,0x230,0x118,0x08c, /* triplet notes     */
  0x6900,0x3480,0x1a40,0x0d20,0x690,0x348,0x1a4,0x0d2, /* dotted triplet    */
  0x5400,0x2a00,0x1500,0x0a80,0x540,0x2a0,0x150,0x0a8, /* quintuplet        */
  0x7e00,0x3f00,0x1f80,0x0fc0,0x7e0,0x3f0,0x1f8,0x0fc, /* dotted quintuplet */
  0x5a00,0x2d00,0x1680,0x0b40,0x5a0,0x2d0,0x168,0x0b4, /* septuplet         */
  0x8700,0x4380,0x21c0,0x10e0,0x870,0x438,0x21c,0x10e  /* dotted septuplet  */
};

UWORD CDurations[64];

/* DEFAULT_BASE_DURATION = Durations[4] */
UWORD BaseDuration = DEFAULT_BASE_DURATION;
UWORD BaseRange    = 1;
UWORD SmusTempo;
UWORD TempoFactor;
char syscomm[300];

/* *********************************************************************** */       
LOCAL FILE *fout; /* to put the results */
/* *********************************************************************** */       
/* return the duration in 128th (0 if non tonic chord note)                */
UWORD Duration(sn)
SMNote sn;
{
  UWORD dur;
  if (sn.chord) return(0);
  dur = Durations[sn.duration];
  return(dur); 
}

/* *********************************************************************** */
UWORD STDuration(sn)
SMNote sn;
{
  return(CDurations[sn.duration]);
}
/* *********************************************************************** */
/* advance the track with one event, return TRUE if track is finnished     */
BOOL AdvanceTrack(clock,tr)
ULONG clock;
struct Track *tr;
{

  tr->tr_pos++; /* advance on event */
  if (tr->tr_pos >= tr->tr_len) { /* track is finnished */
    tr->tr_playing = FALSE;
    printf("TRACK %d END at POS %d\n",tr->tr_num,clock);
    return(FALSE);
  }
  return(TRUE);
}

/* *********************************************************************** */
WORD EventCase(event)
SNote event; /* type is SNote for facility (need sID)*/
{
  /*in this proc, .tone is used as .sID */

  if (AsDuration(event.tone)) {
    if (event.tone == SID_Rest) return(REST);
    if (event.chord) return(CHORD);
    return(NOTE);
  }
  else switch (event.tone) {
   case SID_Instrument : return(SETI);
   case SID_Dynamic    : return(DYNA);
   case SID_Tempo      : return(TEMP);
   case SID_TimeSig    : return(TSIG);
   case SID_KeySig     : return(KSIG);
   case SID_Clef       : return(CLEF);
   case SID_MIDI_Chnl  : return(MCHA);
   case SID_MIDI_Preset: return(MPRE);
   default             : break;
  }
  return(UNKN);
}
/* *********************************************************************** */
PlayNotes(clock,note,tr)
ULONG clock;
SNote note;
struct Track *tr;
{
  SEvent sc;
  WORD dur;
  WORD ic;
  UBYTE tone;
  WORD oct;
  WORD pitch;


  dur = tr->tr_dur;
  ic  = tr->tr_ic;

  if (ic >= InstrList->its_num) { /* non defined intruments */
    ic = InstrList->its_num; /* use default */
  }
  if (ic == NO_INSTR) {  /* instrumentless notes */
    ic = InstrList->its_num; /* use default */
  }
  tone  = note.tone;
  oct   = ((WORD) tone) / OCT_LEN +3+GTranspose;
  pitch = ((WORD) tone) % OCT_LEN;
  InstrList->its_NumPN[ic]+=1;
  fprintf(fout,"i %d.01 %d %d %d.%02d %d\n",ic+1,clock,dur,oct,pitch,tr->tr_vol);
  while(tr->tr_chordp>=0) {
      sc = tr->tr_chord[tr->tr_chordp--];
      tone  = sc.sID;
      oct   = ((WORD) tone) / OCT_LEN +3+GTranspose;
      pitch = ((WORD) tone) % OCT_LEN;
      InstrList->its_NumPN[ic]+=1;
      fprintf(fout,"i %d.01 %d %d %d.%02d %d\n",ic+1,clock,dur,oct,pitch,tr->tr_vol);
  }
}
/* *********************************************************************** */
VOID PlayTrackNote(clock,tr)
ULONG clock;
struct Track *tr;
{
  SEvent event;
  SEvent  tien;
  SNote  *ptien;
  BOOL scanning;
  UWORD newtempo;

  tr->tr_dur--;
  if (tr->tr_dur>0) return;

  tr->tr_dur  = 0;
  tr->tr_chordp = -1;

  scanning = TRUE;
  while(scanning) {
   event = tr->tr_fse[tr->tr_pos]; /* read current event */
   scanning = AdvanceTrack(clock,tr);
   switch (EventCase(event)) {

    case REST  : tr->tr_dur = STDuration(event);
                 scanning = FALSE;
                 break;

    case CHORD : if (tr->tr_chordp+1<MAX_CHORD)
                  tr->tr_chordp++;
                  tr->tr_chord[tr->tr_chordp] = event; /* store note */
                 break;
    case NOTE  : /* event is a note */
                 scanning = FALSE;             /* we found something to play */
                 tr->tr_dur = STDuration(event); /* get duration */
                 tien = event;
                 ptien = (SNote *) &tien;
                 while(ptien->tieOut) { /* tied notes */
                   tien = tr->tr_fse[tr->tr_pos++];
                   if (ptien->chord!=1) tr->tr_dur += STDuration(tien);
                 }
                 PlayNotes(clock,event,tr); /* write out note(s) */
                 break;

    case SETI  : if (event.data < InstrList->its_num) {
                   tr->tr_ic = IRegs[event.data]; /* change current instrument */
                 }
                 break;

    case DYNA  : tr->tr_vol = (UWORD)(event.data);  /* change volume */
                 break;

    case TEMP  : newtempo = ((UWORD) event.data)*TempoFactor;
                 fprintf(fout,"s\n;Change tempo to %d q/min \n",event.data);
                 fprintf(fout,"t 0 %d\n",newtempo);
                 break;

    case TSIG  :
    case KSIG  :
    case CLEF  :
    case MCHA  :
    case MPRE  : break; /* not handle */

    case UNKN  :
    default    : /* printf("UNKNOW SEvent (%d)\n",event.sID); */
                 break; /* others not handle */
   } /* switch */
  } /* while (scanning) */

}
/* *********************************************************************** */
SMUS_To_CSOUND(outfn)
char *outfn;
{
  struct Track *tr;
  UWORD ntr;
  ULONG clock;
  UWORD ntc;
  WORD ic,io;
  ULONG lastpos;
  char *scorefn[300];

   strcpy(scorefn,outfn);
   strcat(scorefn,".sco");
  /* *************************************************************************** */
  /* should use process ID for security */
  fout = GOpen("/tmp/SMUS_2_SCO_TMP_FILE",MODE_NEWFILE);
  if (fout==NULL) {
     fprintf(stderr,"could'nt create temporary file /tmp/SMUS_2_SCO_TMP_FILE\n");
     exit();
  }

  ntc = (UWORD)The_Score->sc_ntc;
  for(ntr=0;ntr<ntc;ntr++) {
    tr = The_Score->sc_tracks[ntr];
    tr->tr_num       = ntr;
    tr->tr_pos       = 0;
    tr->tr_ic        = (DefTI==TRUE) ? IRegs[ntr]:NO_INSTR;
    tr->tr_dur       = 0;
    tr->tr_vol       = MAX_VOLUME;
    tr->tr_done      = 0;
    tr->tr_playing   = TRUE;
    tr->tr_chordp    = -1;
  }

  lastpos = The_Score->sc_duration;

  printf("converting to file %s\n",scorefn);
  /* *************************************************************************** */
  fprintf(fout,";Score Name : %s\n",The_Score->sc_name);
  if (The_Score->sc_auth != NULL)
    fprintf(fout,";Author Name : %s\n",The_Score->sc_auth);
  if (The_Score->sc_copyr != NULL)
    fprintf(fout,";(c) : %s\n",The_Score->sc_copyr);
  if (The_Score->sc_anno != NULL)
    fprintf(fout,";Note : %s\n",The_Score->sc_anno);
  fprintf(fout,";created by Smus2St\n\n");
  /* *************************************************************************** */
  fprintf(fout,";Instruments assignement :\n");
  for(ic=0;ic<InstrList->its_num;ic++) {
    if (InstrList->its_Used[ic] == TRUE) {
      fprintf(fout,"; %d <- %s\n",ic+1,InstrList->its_instrs[ic]->name);
    }
  }
  fprintf(fout,"; %d <- Default Instrument\n\n",InstrList->its_num+1);

  /* *************************************************************************** */
  TempoFactor = (Durations[2]/BaseDuration)*BaseRange;
  SmusTempo = The_Score->sc_schd->tempo >> 7;
  fprintf(fout,";TEMPO : %d q/min\n",SmusTempo);
  fprintf(fout,"t 0 %d\n",SmusTempo*TempoFactor);

  /* *************************************************************************** */
  for(clock=0;clock<lastpos;clock++) { /* main loop */
    for(ntr=0;ntr<ntc;ntr++) {         /* for each track */
      tr = The_Score->sc_tracks[ntr];
      if (tr->tr_playing==TRUE) PlayTrackNote(clock,tr); /* play note(s) from it */
    }
  }
  for(ntr=0;ntr<ntc;ntr++) {         /* for each track */
    tr = The_Score->sc_tracks[ntr];
    if (tr->tr_playing==TRUE) 
     printf("track %d : pos = %d end = %d\n",ntr,tr->tr_pos,tr->tr_len);
  }
  /* *************************************************************************** */
  fprintf(fout,"s\ne\n");
  fprintf(fout,";Instruments usage :\n");
  for(ic=0;ic<InstrList->its_num;ic++) {
    if (InstrList->its_Used[ic] == TRUE) {
      fprintf(fout,"; %s : %d notes\n",InstrList->its_instrs[ic]->name,InstrList->its_NumPN[ic]);
    }
  }
  fprintf(fout,"; Default Instrument : %d notes\n\n",InstrList->its_NumPN[InstrList->its_num]);
  ic = InstrList->its_num;
  if (InstrList->its_NumPN[ic] != 0) {
    InstrList->its_Used[ic] = TRUE;
    InstrList->its_instrs[ic] = AllocObject(RefInstrument);
    strcpy(InstrList->its_instrs[ic]->name,"default_instrument");
  }
  else InstrList->its_Used[ic] = FALSE;
  /* *************************************************************************** */
  GClose(fout);

  /* *************************************************************************** */
  fout = GOpen(scorefn,MODE_NEWFILE);
  if (fout==NULL) {
     fprintf(stderr,"could'nt open file %s\n",scorefn);
     exit();
  }
  /* *************************************************************************** */
  if (ScoreOnly == FALSE) {
    GenSoundOpen(outfn);
    for(ic=0;ic<=InstrList->its_num;ic++) {
      if (InstrList->its_Used[ic] == TRUE) {
        GenSound(InstrList->its_instrs[ic]->name,ic);
        for(io=0;io<ctoct;io++) {
          fprintf(fout,"f%d 0 %d 1 %d 0\n",ic*MOT2+io+1,UpPow2(oneshot[io]),ic*MOT2+io+1);
          if (repeat[io]!=0)
            fprintf(fout,"f%d 0 %d 1 %d 0\n",ic*MOT2+io+MAX_OCT+1,UpPow2(repeat[io]),ic*MOT2+io+MAX_OCT+1);
        }
      }   
    }
    GenSoundClose();
    GClose(fout);
  }
  /* *************************************************************************** */
  sprintf(syscomm,"cat /tmp/SMUS_2_SCO_TMP_FILE >>%s",scorefn);
  system (syscomm);
  sprintf(syscomm,"/bin/rm /tmp/SMUS_2_SCO_TMP_FILE");
  system (syscomm);

}
/* *********************************************************************** */
