/* *********************************************************************** */ 
/*                         Smus2sco.c v2.0                                 */
/*                       1990 Jean-Francois Giorgi                         */
/* *********************************************************************** */ 

#ifndef SMUS2SCO_H
#define SMUS2SCO_H
/* *********************************************************************** */
#include <iff/smus.h>
#include <iff/gio.h>
#include <iff/compiler.h>
#include <exec/types.h>

/* *********************************************************************** */

#define OCT_LEN  12               /* an octave   = 12 notes       */
#define TWO_OCT  2*OCT_LEN        /* two octaves = 2 * one octave */
#define MAX_VOLUME 127
/* *********************************************************************** */
typedef struct {
  unsigned
    tone     :8,  /* MIDI tone number 0 to 127; 128 = rest */
    chord    :1,  /* 1 = a chorded note */
    tieOut   :1,  /* 1 = tied to the next note or chord */
    duration :6;  /* merging of division,dot,nTuplet */
} SMNote;
/* ************************************************************************** */
typedef LONG Fixed;
/* ************************************************************************** */
#define IsNote(n) ((n >= SID_FirstNote) && (n <= SID_LastNote))
#define AsDuration(n) (IsNote(n) || (n==SID_Rest))
/* ************************************************************************** */
#define ID_BAD   -1L
#define roundup(a) ((a+1L) & ~1L)
#define NO_INSTR -1
/* ************************************************************************** */
/* See EventCase() */
#define UNKN -1
#define REST  1
#define CHORD 2
#define NOTE  3
#define SETI  4
#define DYNA  5
#define TEMP  6
#define TSIG  7
#define KSIG  8
#define CLEF  9
#define MCHA 10
#define MPRE 11
/* ************************************************************************** */
/* internal format */
/* ************************************************************************* */

#define MAX_CHORD 20

struct Track {
  ULONG  tr_len;             /* length of tr_fse                      */
  ULONG  tr_duration;        /* tr_duration in 128th                  */
  UWORD  tr_shnote;          /* shortest note duration                */
  UWORD  tr_num;             /* number of the track                   */
  LONG   tr_pos;             /* current position in tr_fse            */
  WORD   tr_ic;              /* current instrument                    */
  WORD   tr_dur;              /* duration before next Event (backup)   */
  UWORD  tr_vol;             /* current volume                        */
  UWORD  tr_done;            /* != 0 if note(s) are playing           */
  BOOL   tr_playing;         /* TRUE if tr_pos <tr_len                */
  WORD   tr_chordp;          /* length of tr_chord (and index in it)  */
  SEvent tr_chord[MAX_CHORD];/* saved minors notes of a chord         */
  SEvent tr_fse[1];          /* WARNING: tr_fse must be word aligned  */
};
#define MIN_SZ_TRACK ((LONG)(sizeof(struct Track) - sizeof(SEvent)))
/* ************************************************************************** */
struct Score {
  char         *sc_name;                /* name of the score     */
  char         *sc_auth;                /* author name           */
  char         *sc_anno;                /* author's text annot.  */
  char         *sc_copyr;               /* copyright notice      */
  UBYTE        sc_ntc;                  /* number of tracks      */ 
  SScoreHeader *sc_schd;                /* smus header           */ 
  ULONG        sc_duration;             /* smus duration         */ 
  struct Track *sc_tracks[1];           /* the tracks            */
};
/* size of struct Score for a given number (nt) of tracks */
#define SZ_SCORE(nt) ((LONG)(sizeof(struct Score)+((nt-1)*sizeof(struct Track *))))  
/* ************************************************************************** */
#define MAX_INST 256

struct Instruments {
  WORD          its_num;                   /* number of instruments           */
  WORD          its_cour;                  /* current instrument              */
  LONG          its_NumPN[MAX_INST];       /* number of played notes          */
  BOOL          its_Used[MAX_INST];        /* instruments usage               */
  UBYTE         its_MinTone[MAX_INST];     /* lowest note                     */
  UBYTE         its_MaxTone[MAX_INST];     /* highest note                    */
  RefInstrument *its_instrs[MAX_INST];     /* original SMUS instrument        */
};
#define REFI_SIZE ((LONG)(sizeof(RefInstrument)))

extern WORD IRegs[MAX_INST];

/* ************************************************************************** */
/* forward definitions of globals datas and procedures                        */
/* ************************************************************************** */

extern char *calloc();
extern char *BaseName();

extern BOOL DefTI;
extern BOOL ScoreOnly;
extern BOOL AnalOnly;
extern WORD GTranspose;

extern UWORD Durations[64];
extern UWORD CDurations[64];
#define DEFAULT_BASE_DURATION 0x690
extern UWORD BaseDuration;
extern UWORD BaseRange;
extern UWORD shnote;
#define LONG_NOTE 4
#define MINDUR(a,b)  (Durations[a]<Durations[b]?(a):(b))

extern struct Score *The_Score;
extern struct Instruments *InstrList;

#define AllocStruct(t) ((struct t *)calloc((LONG)sizeof(struct t),1))
#define AllocObject(t) ((t *)calloc((LONG)sizeof(t),1))
#define AllocString(s) ((char *)calloc((LONG)s,1))

#define MAX_OCT 10
#define MOT2 (MAX_OCT*2)

extern ULONG oneshot[MAX_OCT],repeat[MAX_OCT];
extern UBYTE ctoct;       /* # of octaves of waveforms */
extern ULONG UpPow2(); /* see gensound.c */

#endif /* SMUS2SCO_H */

